;;; -*- mode: LISP; package: CLONES -*-

(in-package :clones)


(defclass score ()
  ((amount :initarg :amount
           :accessor amount
           :initform 0)
   (date :initarg :date
         :accessor date
         :initform (get-universal-time))
   (name :initarg :name
         :accessor name
         :initform "")))

(defun make-score-from-string (string)
  "STRING is of form: <score> <date> <name>"
  (ignore-errors
    (let* ((first (position #\space string))
           (second (position #\space string :start (1+ first))))
      (make-instance 'score
                     :amount (parse-integer string :end first)
                     :date (parse-integer string :start (1+ first) :end second)
                     :name (subseq string (1+ second))))))

(defun read-scores (&optional (file *scores-file-name*))
  (with-open-file (in file :direction :input :if-does-not-exist nil)
    (when in
      (loop for line = (read-line in nil nil)
            while line
            when (make-score-from-string line)
            collect it))))


(defun write-scores (scores &optional (file *scores-file-name*))
  (ensure-directories-exist file)
  (with-open-file (out file :direction :output
                       :if-does-not-exist :create
                       :if-exists :supersede)
    (let ((scores (sort (copy-list scores) #'> :key #'amount)))
      (loop for score in scores
            repeat 5
            do (with-slots (amount date name) score
                 (format out "~D ~D ~A~%" amount date name))
            collecting score))))
