;;; -*- mode: LISP; package: CLONES.CLIM -*-

(in-package #:clones.clim)


(defvar *background-ink* +black+ ;;+grey20+
  )


(defclass theme ()
  ((cell-width :initarg :cell-width
               :accessor cell-width
               :initform *cell-width*)
   (cell-height :initarg :cell-height
                :accessor cell-height
                :initform *cell-height*)))


(defgeneric draw-cell (theme cell stream)
  (:documentation "Draws CELL to STREAM using THEME."))


(defgeneric initialize-theme (theme pane)
  (:documentation "Called when the frame is set up to do theme setup.")
  (:method (theme pane)
           "Default method that does nothing."
           (declare (ignore theme pane))
           nil))



;;; Simple drawn themes

;;; Helper function for simple drawn themes.
;;; 
(defun color-ink (color)
  (ecase color
    (:empty    *background-ink*)
    (:red      +red3+)
    (:green    +green3+)
    (:blue     +blue3+)
    (:red-hl   +red1+)
    (:green-hl +green1+)
    (:blue-hl  +blue1+)))


(defclass rectangle-theme (theme)
  ())


(defmethod draw-cell ((theme rectangle-theme) cell stream)
  (with-slots (cell-width cell-height) theme
    (with-slots (x y clones::color) cell
      (let* ((x1 (* x cell-width))
             (y1 (* y cell-height))
             (x2 (+ x1 cell-width))
             (y2 (+ y1 cell-width)))
        (draw-rectangle* stream x1 y1 x2 y2
                         :filled t
                         :ink (color-ink clones::color))))))




(defclass circle-theme (theme)
  ((cx-offset ;; :initarg :cx-offset
              :type fixnum
              :initform 0)
   (cy-offset ;; :initarg :cy-offset
              :type fixnum
              :initform 0)
   (radius ;; :initarg :radius
           :type fixnum
           :initform 0)))



(defmethod initialize-instance :after ((object circle-theme) &rest initargs)
  (declare (ignore initargs))
  (with-slots (cell-width cell-height cx-offset cy-offset radius) object
    (setf cx-offset (truncate cell-width 2)
          cy-offset (truncate cell-width 2)
          radius (min cx-offset cy-offset))))


(defmethod draw-cell ((theme circle-theme) cell stream)
  (with-slots (cell-width cell-height cx-offset cy-offset radius) theme
    (with-slots (x y) cell
      (let ((x1 (* x cell-width))
            (y1 (* y cell-height)))
        (draw-circle* stream (+ x1 cx-offset) (+ y1 cy-offset) radius
                      :filled t
                      :ink (color-ink (cell-color cell)))))))


